@@ -29,6 +29,8 @@
 
 #include "soprano_export.h"
 #include "literalvalue.h"
+#include "languagetag.h"
+#include "sopranomacros.h"
 
 namespace Soprano
 {
@@ -41,25 +43,21 @@ namespace Soprano
      * is essentially a set of Statements.
      *
      * A Node can have one of four types: EmptyNode, ResourceNode, LiteralNode, and BlankNode.
-     * Resource nodes are identified through their URI (uri()), literal nodes have a LiteralValue (literal())
-     * and an optional language string (language()), and blank nodes have a string identifier.
+     * Resource nodes are identified through their URI (uri()), literal nodes have a LiteralValue (literal()),
+     * and blank nodes have a string identifier.
      *
      * Empty nodes can be used as wildcards in methods such as Model::listStatements.
      *
-     * \warning Be aware that string literals in %Soprano always have type
-     * xsd:string. There is no direct support for rdfs:Literal. Backend implementations
-     * should honour this restriction.
-     *
      * \author Daniele Galdi <daniele.galdi@gmail.com><br>Sebastian Trueg <trueg@kde.org>
      */
     class SOPRANO_EXPORT Node
     {
     public:
         enum Type {
-            EmptyNode    = 0, /**< An empty node, can be used as a placeholder in commands like Model::listStatements. */
+            EmptyNode    = 0, /**< An empty node, can be used as a wildcard in commands like Model::listStatements. */
             ResourceNode = 1, /**< A resource node has a URI which can be accessed via uri() */
             LiteralNode  = 2, /**< A literal node has a literal value and an optional language. */
-            BlankNode    = 3  /**< blank node has an identifier string */
+            BlankNode    = 3  /**< A blank node has an identifier string */
         };
 
         /**
@@ -95,6 +93,18 @@ namespace Soprano
          */
         explicit Node( const QString& id );
 
+        /**
+         * Creates a literal node.
+         *
+         * \param value The value of a node. If empty the node will become
+         *              an empty node.
+         *
+         * \sa createLiteralNode()
+         *
+         * \since 2.3
+         */
+        Node( const LiteralValue& value );
+
         /**
          * Creates a literal node.
          *
@@ -103,10 +113,16 @@ namespace Soprano
          * \param language The language of the literal value.
          *
          * \sa createLiteralNode()
+         *
+         * \deprecated Use Soprano::Node::Node( const LiteralValue& ) and
+         *             Soprano::LiteralValue::createPlainLiteral( const QString&, const LanguageTag& )
          */
-        Node( const LiteralValue& value,
-              const QString& language = QString() );
+        SOPRANO_CONSTRUCTOR_DEPRECATED Node( const LiteralValue& value,
+                                             const QString& language );
 
+        /**
+         * Copy constructor.
+         */
         Node( const Node &other );
 
         ~Node();
@@ -119,24 +135,57 @@ namespace Soprano
         Node& operator=( const Node& other );
 
         /**
-         * Assigns resource to this node and makes it a ResourceNode.
+         * Assigns \p resource to this node and makes it a ResourceNode.
          */
         Node& operator=( const QUrl& resource );
 
+        /**
+         * Assigns \p literal to this node and makes it a LiteralNode.
+         */
         Node& operator=( const LiteralValue& literal );
 
+        /**
+         * Comparision operator.
+         * \return \p true if this node and \p other are equal.
+         */
         bool operator==( const Node& other ) const;
+
+        /**
+         * Comparision operator.
+         * \return \p true if this node and \p other differ.
+         */
         bool operator!=( const Node& other ) const;
 
-        bool operator==( const QUrl& other ) const;
+        /**
+         * Comparision operator.
+         * \return \p true if this node is a ResourceNode and
+         * has URI \p uri.
+         */
+        bool operator==( const QUrl& uri ) const;
+
+        /**
+         * Comparision operator.
+         * \return \p true if this node is a LiteralNode and
+         * has literal value \p other.
+         */
         bool operator==( const LiteralValue& other ) const;
 
         /**
-         * Match this node against other. The only difference
+         * Match this node against template node \a other. The only difference
          * to operator== is that empty nodes are matched as wildcards,
          * i.e. they match any other node.
          *
-         * \return true if this node matches other, false if not.
+         * Be aware that the following is NOT always true since only \a other
+         * is treated a a wildcard:
+         *
+         * \code
+         * // NOT always true:
+         * a.matches(b) == b.matches(a)
+         * \endcode
+         *
+         * \return \p true if this node matches other, \p false if not.
+         *
+         * \sa Statement::matches
          */
         bool matches( const Node& other ) const;
         //@}
@@ -146,32 +195,32 @@ namespace Soprano
          */
         //@{
         /**
-         * \return The Node type.
+         * \return The node type.
          */
         Type type() const;
 
         /**
-         * \return true if the Node is empty.
+         * \return \p true if the node is empty.
          */
         bool isEmpty() const;
 
         /**
-         * \return true if the Node is a Resource,Literal or Blank.
+         * \return \p true if the node is a ResourceNode, LiteralNode or BlankNode.
          */
         bool isValid() const ;
 
         /**
-         * \return true if the Node is a Literal.
+         * \return \p true if the node is a LiteralNode.
          */
         bool isLiteral() const;
 
         /**
-         * \return true if the Node is a Resource.
+         * \return \p true if the node is a ResourceNode.
          */
         bool isResource() const;
 
         /**
-         * \return true if the Node is a Blank node (anonymous).
+         * \return \p true if the node is a BlankNode (anonymous).
          */
         bool isBlank() const;
         //@}
@@ -181,7 +230,7 @@ namespace Soprano
          */
         //@{
         /**
-         * \return The URI if the node is a Resource node.
+         * \return The URI if the node is a ResourceNode.
          *         An null QUrl otherwise.
          */
         QUrl uri() const;
@@ -193,7 +242,7 @@ namespace Soprano
         //@{
         /**
          * Retrieve a blank node's identifier.
-         * \return The node's identifier if it is a blank node, a null
+         * \return The node's identifier if it is a BlankNode, a null
          * string otherwise.
          */
         QString identifier() const;
@@ -204,14 +253,14 @@ namespace Soprano
          */
         //@{
         /**
-         * \return The Literal value if the node is a Literal node.
+         * \return The literal value if the node is a LiteralNode.
          *         An null QString otherwise.
          */
         LiteralValue literal() const;
 
         /**
          * \return The datatype URI of a literal node, i.e. the XML schema type
-         *         or an empty value if the node is not a literal.
+         *         or an empty value if the node is not a LiteralNode.
          * \sa LiteralValue::dataTypeUri
          */
         QUrl dataType() const;
@@ -223,10 +272,16 @@ namespace Soprano
          *
          * \return A string representing the language of the literal value
          *         or an empty string if the node is not a literal.
+         *
+         * \deprecated Language exists on the Soprano::LiteralValue. Use Soprano::Node::literal() and
+         *             Soprano::LiteralValue::language().
          */
         QString language() const;
         //@}
 
+        /**
+         * \name Conversion
+         */
         //@{
         /**
          * Converts the Node to a string.
@@ -241,6 +296,14 @@ namespace Soprano
         /**
          * Convert a Node into N3 notation to be used in SPARQL graph patterns.
          *
+         * Examples:
+         * \code
+         * <http://soprano.sourceforce.net/>
+         * "Hello World"^^<http://www.w3.org/2001/XMLSchema#string>
+         * "09-08-1977T17:42.234Z"^^<http://www.w3.org/2001/XMLSchema#dateTime>
+         * _:blankNode
+         * \endcode
+         *
          * \return A string representing the node in N3 encoding or an empty
          * string for invalid nodes.
          *
@@ -250,7 +313,7 @@ namespace Soprano
          */
         QString toN3() const;
         //@}
-        
+
         /**
          * Convenience method to create an empty node.
          * Using this method instead of the default constructor
@@ -267,7 +330,7 @@ namespace Soprano
          *
          * \param uri The URI of the node. If empty the type will be ignored
          *            and an empty node will be created.
-         * 
+         *
          * \return A resource Node or an empty Node if the specified URI is empty.
          */
         static Node createResourceNode( const QUrl& uri );
@@ -288,6 +351,21 @@ namespace Soprano
          */
         static Node createBlankNode( const QString& id );
 
+        /**
+         * Convenience method to create a literal node.
+         * Using this method instead of the constructor
+         * may result in better readable code.
+         *
+         * \param value The value of a node. If empty the node will become
+         *              an empty node.
+         *
+         * \return A literal node or an empty node if the specified value
+         * was empty.
+         *
+         * \since 2.3
+         */
+        static Node createLiteralNode( const LiteralValue& value );
+
         /**
          * Convenience method to create a literal node.
          * Using this method instead of the constructor
@@ -299,8 +377,63 @@ namespace Soprano
          *
          * \return A literal node or an empty node if the specified value
          * was empty.
+         *
+         * \deprecated Use Soprano::Node::createLiteralNode( const LiteralValue& ) and
+         *             Soprano::LiteralValue::createPlainLiteral( const QString&, const LanguageTag& )
          */
-        static Node createLiteralNode( const LiteralValue& value, const QString& language );
+        static SOPRANO_DEPRECATED Node createLiteralNode( const LiteralValue& value, const QString& language );
+
+        /**
+         * Format a resource URI as N3 string to be used in SPARQL queries.
+         *
+         * \return A string representing the resource in N3 encoding or an empty
+         * string for invalid URIs.
+         *
+         * Example:
+         * \code
+         * <http://soprano.sourceforce.net/>
+         * \endcode
+         *
+         * \sa toN3
+         *
+         * \since 2.3
+         */
+        static QString resourceToN3( const QUrl& resource );
+
+        /**
+         * Format a blank node identifier as N3 string to be used in SPARQL queries.
+         *
+         * \return A string representing the blank identifier in N3 encoding or an empty
+         * string for invalid/empty ids.
+         *
+         * Example:
+         * \code
+         * _:blankNode
+         * \endcode
+         *
+         * \sa toN3
+         *
+         * \since 2.3
+         */
+        static QString blankToN3( const QString& blank );
+
+        /**
+         * Format a literal value as N3 string to be used in SPARQL queries.
+         *
+         * \return A string representing the literal in N3 encoding or an empty
+         * string for invalid literals.
+         *
+         * Examples:
+         * \code
+         * "Hello World"^^<http://www.w3.org/2001/XMLSchema#string>
+         * "09-08-1977T17:42.234Z"^^<http://www.w3.org/2001/XMLSchema#dateTime>
+         * \endcode
+         *
+         * \sa toN3
+         *
+         * \since 2.3
+         */
+        static QString literalToN3( const LiteralValue& literal );
 
     private:
         class NodeData;
@@ -310,9 +443,15 @@ namespace Soprano
         QSharedDataPointer<NodeData> d;
     };
 
+    /**
+     * \relates Soprano::Node
+     */
     SOPRANO_EXPORT uint qHash( const Node& node );
 }
 
+/**
+ * \relates Soprano::Node
+ */
 SOPRANO_EXPORT QDebug operator<<( QDebug s, const Soprano::Node& );
 
 /**
@@ -320,9 +459,13 @@ SOPRANO_EXPORT QDebug operator<<( QDebug s, const Soprano::Node& );
  * based on the N-Triples standard, except that it uses Unicode strings.
  *
  * \sa Soprano::Node::toN3()
+ *
+ * \relates Soprano::Node
  */
 SOPRANO_EXPORT QTextStream& operator<<( QTextStream& s, const Soprano::Node& );
 
+#if QT_VERSION < 0x040700
 SOPRANO_EXPORT uint qHash( const QUrl& url );
+#endif
 
 #endif // SOPRANO_NODE_H
